@@ -1,6 +1,6 @@
-import io
+import asyncio
 from pathlib import Path
-from typing import List, Union
+from typing import IO, Any, List, Union
 
 from agno.document.base import Document
 from agno.document.reader.base import Reader
@@ -15,13 +15,16 @@ except ImportError:
 class DocxReader(Reader):
     """Reader for Doc/Docx files"""
 
-    def read(self, file: Union[Path, io.BytesIO]) -> List[Document]:
+    def read(self, file: Union[Path, IO[Any]]) -> List[Document]:
+        """Read a docx file and return a list of documents"""
         try:
             if isinstance(file, Path):
+                if not file.exists():
+                    raise FileNotFoundError(f"Could not find file: {file}")
                 log_info(f"Reading: {file}")
                 docx_document = DocxDocument(str(file))
                 doc_name = file.stem
-            else:  # Handle file-like object from upload
+            else:
                 log_info(f"Reading uploaded file: {file.name}")
                 docx_document = DocxDocument(file)
                 doc_name = file.name.split(".")[0]
@@ -35,12 +38,22 @@ class DocxReader(Reader):
                     content=doc_content,
                 )
             ]
+
             if self.chunk:
                 chunked_documents = []
                 for document in documents:
                     chunked_documents.extend(self.chunk_document(document))
                 return chunked_documents
             return documents
+
         except Exception as e:
             logger.error(f"Error reading file: {e}")
             return []
+
+    async def async_read(self, file: Union[Path, IO[Any]]) -> List[Document]:
+        """Asynchronously read a docx file and return a list of documents"""
+        try:
+            return await asyncio.to_thread(self.read, file)
+        except Exception as e:
+            logger.error(f"Error reading file asynchronously: {e}")
+            return []
