@@ -79,8 +79,6 @@ class Memory:
     memories: Optional[Dict[str, Dict[str, UserMemory]]] = None
     # Manager to manage memories
     memory_manager: Optional[MemoryManager] = None
-    delete_memories: bool = True
-    clear_memories: bool = True
 
     # Session summaries per session per user
     summaries: Optional[Dict[str, Dict[str, SessionSummary]]] = None
@@ -95,6 +93,11 @@ class Memory:
     # Team context per session
     team_context: Optional[Dict[str, TeamContext]] = None
 
+    # Whether to delete memories
+    delete_memories: bool = False
+    # Whether to clear memories
+    clear_memories: bool = False
+
     debug_mode: bool = False
     version: int = 2
 
@@ -108,6 +111,8 @@ class Memory:
         summaries: Optional[Dict[str, Dict[str, SessionSummary]]] = None,
         runs: Optional[Dict[str, List[Union[RunResponse, TeamRunResponse]]]] = None,
         debug_mode: bool = False,
+        delete_memories: bool = False,
+        clear_memories: bool = False,
     ):
         self.memories = memories or {}
         self.summaries = summaries or {}
@@ -115,6 +120,9 @@ class Memory:
 
         self.debug_mode = debug_mode
 
+        self.delete_memories = delete_memories
+        self.clear_memories = clear_memories
+
         self.model = model
 
         if self.model is not None and isinstance(self.model, str):
@@ -144,9 +152,6 @@ class Memory:
 
         self.debug_mode = debug_mode
 
-        if self.db is not None:
-            self.hydrate_from_db()
-
     def set_model(self, model: Model) -> None:
         if self.memory_manager is None:
             self.memory_manager = MemoryManager(model=deepcopy(model))
@@ -170,23 +175,29 @@ class Memory:
             self.model = OpenAIChat(id="gpt-4o")
         return self.model
 
-    def hydrate_from_db(self):
+    def refresh_from_db(self, user_id: Optional[str] = None):
         if self.db:
-            all_memories = self.db.read_memories()
+            # If no user_id is provided, read all memories
+            if user_id is None:
+                all_memories = self.db.read_memories()
+            else:
+                all_memories = self.db.read_memories(user_id=user_id)
             # Reset the memories
             self.memories = {}
             for memory in all_memories:
                 if memory.user_id is not None and memory.id is not None:
                     self.memories.setdefault(memory.user_id, {})[memory.id] = UserMemory.from_dict(memory.memory)
 
-    def initialize(self):
+    def set_log_level(self):
         if self.debug_mode or getenv("AGNO_DEBUG", "false").lower() == "true":
             self.debug_mode = True
             set_log_level_to_debug()
         else:
             set_log_level_to_info()
 
-        self.hydrate_from_db()
+    def initialize(self, user_id: Optional[str] = None):
+        self.set_log_level()
+        self.refresh_from_db(user_id=user_id)
 
     def to_dict(self) -> Dict[str, Any]:
         _memory_dict = {}
@@ -219,26 +230,44 @@ class Memory:
         return _memory_dict
 
     # -*- Public Functions
-    def get_user_memories(self, user_id: str) -> List[UserMemory]:
+    def get_user_memories(self, user_id: Optional[str] = None, refresh_from_db: bool = True) -> List[UserMemory]:
         """Get the user memories for a given user id"""
+        if user_id is None:
+            user_id = "default"
+        # Refresh from the DB
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
         if self.memories is None:
             return []
         return list(self.memories.get(user_id, {}).values())
 
-    def get_session_summaries(self, user_id: str) -> List[SessionSummary]:
+    def get_session_summaries(self, user_id: Optional[str] = None) -> List[SessionSummary]:
         """Get the session summaries for a given user id"""
+        if user_id is None:
+            user_id = "default"
         if self.summaries is None:
             return []
         return list(self.summaries.get(user_id, {}).values())
 
-    def get_user_memory(self, user_id: str, memory_id: str) -> Optional[UserMemory]:
+    def get_user_memory(
+        self, memory_id: str, user_id: Optional[str] = None, refresh_from_db: bool = True
+    ) -> Optional[UserMemory]:
         """Get the user memory for a given user id"""
+        if user_id is None:
+            user_id = "default"
+        # Refresh from the DB
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
         if self.memories is None:
             return None
         return self.memories.get(user_id, {}).get(memory_id, None)
 
-    def get_session_summary(self, user_id: str, session_id: str) -> Optional[SessionSummary]:
+    def get_session_summary(self, session_id: str, user_id: Optional[str] = None) -> Optional[SessionSummary]:
         """Get the session summary for a given user id"""
+
+        if user_id is None:
+            user_id = "default"
         if self.summaries is None:
             return None
         return self.summaries.get(user_id, {}).get(session_id, None)
@@ -247,6 +276,7 @@ class Memory:
         self,
         memory: UserMemory,
         user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
     ) -> str:
         """Add a user memory for a given user id
         Args:
@@ -257,14 +287,16 @@ class Memory:
         """
         from uuid import uuid4
 
-        self.initialize()
-
         memory_id = memory.memory_id or str(uuid4())
         if memory.memory_id is None:
             memory.memory_id = memory_id
         if user_id is None:
             user_id = "default"
 
+        # Refresh from the DB
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
         if not memory.last_updated:
             memory.last_updated = datetime.now()
 
@@ -286,6 +318,7 @@ class Memory:
         memory_id: str,
         memory: UserMemory,
         user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
     ) -> Optional[str]:
         """Replace a user memory for a given user id
         Args:
@@ -296,11 +329,13 @@ class Memory:
             str: The id of the memory
         """
 
-        self.initialize()
-
         if user_id is None:
             user_id = "default"
 
+        # Refresh from the DB
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
         if not memory.last_updated:
             memory.last_updated = datetime.now()
 
@@ -321,13 +356,27 @@ class Memory:
 
         return memory_id
 
-    def delete_user_memory(self, user_id: str, memory_id: str) -> None:
+    def delete_user_memory(
+        self,
+        memory_id: str,
+        user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
+    ) -> None:
         """Delete a user memory for a given user id
         Args:
-            user_id (str): The user id to delete the memory from
             memory_id (str): The id of the memory to delete
+            user_id (Optional[str]): The user id to delete the memory from. If not provided, the memory is deleted from the "default" user.
         """
-        self.initialize()
+        if user_id is None:
+            user_id = "default"
+
+        # Refresh from the DB
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
+        if memory_id not in self.memories[user_id]:  # type: ignore
+            log_warning(f"Memory {memory_id} not found for user {user_id}")
+            return None
 
         del self.memories[user_id][memory_id]  # type: ignore
         if self.db:
@@ -339,7 +388,6 @@ class Memory:
             user_id (str): The user id to delete the memory from
             session_id (str): The id of the session to delete
         """
-        self.initialize()
         del self.summaries[user_id][session_id]  # type: ignore
 
     def get_runs(self, session_id: str) -> List[Union[RunResponse, TeamRunResponse]]:
@@ -355,7 +403,7 @@ class Memory:
         if not self.summary_manager:
             raise ValueError("Summarizer not initialized")
 
-        self.initialize()
+        self.set_log_level()
 
         if user_id is None:
             user_id = "default"
@@ -375,7 +423,7 @@ class Memory:
         if not self.summary_manager:
             raise ValueError("Summarizer not initialized")
 
-        self.initialize()
+        self.set_log_level()
 
         if user_id is None:
             user_id = "default"
@@ -393,9 +441,14 @@ class Memory:
         return session_summary
 
     def create_user_memories(
-        self, message: Optional[str] = None, messages: Optional[List[Message]] = None, user_id: Optional[str] = None
+        self,
+        message: Optional[str] = None,
+        messages: Optional[List[Message]] = None,
+        user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
     ) -> str:
         """Creates memories from multiple messages and adds them to the memory db."""
+        self.set_log_level()
         if not messages and not message:
             raise ValueError("You must provide either a message or a list of messages")
 
@@ -412,27 +465,38 @@ class Memory:
             log_warning("MemoryDb not provided.")
             return "Please provide a db to store memories"
 
-        self.initialize()
-
         if user_id is None:
             user_id = "default"
 
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
         existing_memories = self.memories.get(user_id, {})  # type: ignore
         existing_memories = [
             {"memory_id": memory_id, "memory": memory.memory} for memory_id, memory in existing_memories.items()
         ]
         response = self.memory_manager.create_or_update_memories(  # type: ignore
-            messages=messages, existing_memories=existing_memories, user_id=user_id, db=self.db
+            messages=messages,
+            existing_memories=existing_memories,
+            user_id=user_id,
+            db=self.db,
+            delete_memories=self.delete_memories,
+            clear_memories=self.clear_memories,
         )
 
         # We refresh from the DB
-        self.hydrate_from_db()
+        self.refresh_from_db(user_id=user_id)
         return response
 
     async def acreate_user_memories(
-        self, message: Optional[str] = None, messages: Optional[List[Message]] = None, user_id: Optional[str] = None
+        self,
+        message: Optional[str] = None,
+        messages: Optional[List[Message]] = None,
+        user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
     ) -> str:
         """Creates memories from multiple messages and adds them to the memory db."""
+        self.set_log_level()
         if not messages and not message:
             raise ValueError("You must provide either a message or a list of messages")
 
@@ -449,28 +513,33 @@ class Memory:
             log_warning("MemoryDb not provided.")
             return "Please provide a db to store memories"
 
-        self.initialize()
-
         if user_id is None:
             user_id = "default"
 
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
         existing_memories = self.memories.get(user_id, {})  # type: ignore
         existing_memories = [
             {"memory_id": memory_id, "memory": memory.memory} for memory_id, memory in existing_memories.items()
         ]
 
         response = await self.memory_manager.acreate_or_update_memories(  # type: ignore
-            messages=messages, existing_memories=existing_memories, user_id=user_id, db=self.db
+            messages=messages,
+            existing_memories=existing_memories,
+            user_id=user_id,
+            db=self.db,
+            delete_memories=self.delete_memories,
+            clear_memories=self.clear_memories,
         )
 
         # We refresh from the DB
-        self.hydrate_from_db()
+        self.refresh_from_db()
 
         return response
 
     def update_memory_task(self, task: str, user_id: Optional[str] = None) -> str:
         """Updates the memory with a task"""
-
         if not self.memory_manager:
             raise ValueError("Memory manager not initialized")
 
@@ -481,7 +550,7 @@ class Memory:
         if user_id is None:
             user_id = "default"
 
-        self.initialize()
+        self.refresh_from_db(user_id=user_id)
 
         existing_memories = self.memories.get(user_id, {})  # type: ignore
         existing_memories = [
@@ -498,13 +567,13 @@ class Memory:
         )
 
         # We refresh from the DB
-        self.hydrate_from_db()
+        self.refresh_from_db(user_id=user_id)
 
         return response
 
     async def aupdate_memory_task(self, task: str, user_id: Optional[str] = None) -> str:
         """Updates the memory with a task"""
-
+        self.set_log_level()
         if not self.memory_manager:
             raise ValueError("Memory manager not initialized")
 
@@ -515,7 +584,7 @@ class Memory:
         if user_id is None:
             user_id = "default"
 
-        self.initialize()
+        self.refresh_from_db(user_id=user_id)
 
         existing_memories = self.memories.get(user_id, {})  # type: ignore
         existing_memories = [
@@ -532,7 +601,7 @@ class Memory:
         )
 
         # We refresh from the DB
-        self.hydrate_from_db()
+        self.refresh_from_db(user_id=user_id)
 
         return response
 
@@ -617,7 +686,7 @@ class Memory:
         """Returns the messages from the last_n runs, excluding previously tagged history messages.
         Args:
             session_id: The session id to get the messages from.
-            last_n: The number of runs to return from the end of the conversation.
+            last_n: The number of runs to return from the end of the conversation. Defaults to all runs.
             skip_role: Skip messages with this role.
             skip_history_messages: Skip messages that were tagged as history in previous runs.
         Returns:
@@ -629,7 +698,7 @@ class Memory:
         session_runs = self.runs.get(session_id, [])
         runs_to_process = session_runs[-last_n:] if last_n is not None else session_runs
         messages_from_history = []
-
+        system_message = None
         for run_response in runs_to_process:
             if not (run_response and run_response.messages):
                 continue
@@ -641,8 +710,13 @@ class Memory:
                 # Skip messages that were tagged as history in previous runs
                 if hasattr(message, "from_history") and message.from_history and skip_history_messages:
                     continue
-
-                messages_from_history.append(message)
+                if message.role == "system":
+                    # Only add the system message once
+                    if system_message is None:
+                        system_message = message
+                        messages_from_history.append(system_message)
+                else:
+                    messages_from_history.append(message)
 
         log_debug(f"Getting messages from previous runs: {len(messages_from_history)}")
         return messages_from_history
@@ -668,11 +742,12 @@ class Memory:
         limit: Optional[int] = None,
         retrieval_method: Optional[Literal["last_n", "first_n", "agentic"]] = None,
         user_id: Optional[str] = None,
+        refresh_from_db: bool = True,
     ) -> List[UserMemory]:
         """Search through user memories using the specified retrieval method.
 
         Args:
-            query: The search query for semantic search. Required if retrieval_method is "semantic".
+            query: The search query for agentic search. Required if retrieval_method is "agentic".
             limit: Maximum number of memories to return. Defaults to self.retrieval_limit if not specified. Optional.
             retrieval_method: The method to use for retrieving memories. Defaults to self.retrieval if not specified.
                 - "last_n": Return the most recent memories
@@ -683,13 +758,17 @@ class Memory:
         Returns:
             A list of UserMemory objects matching the search criteria.
         """
-        if not self.memories:
-            return []
 
         if user_id is None:
             user_id = "default"
 
-        self.initialize()
+        self.set_log_level()
+
+        if refresh_from_db:
+            self.refresh_from_db(user_id=user_id)
+
+        if not self.memories:
+            return []
 
         # Use default retrieval method if not specified
         retrieval_method = retrieval_method
@@ -699,7 +778,7 @@ class Memory:
         # Handle different retrieval methods
         if retrieval_method == "agentic":
             if not query:
-                raise ValueError("Query is required for semantic search")
+                raise ValueError("Query is required for agentic search")
 
             return self._search_user_memories_agentic(user_id=user_id, query=query, limit=limit)
 
@@ -723,11 +802,13 @@ class Memory:
                     "schema": MemorySearchResponse.model_json_schema(),
                 },
             }
+            model.structured_outputs = False
         else:
             model.response_format = {"type": "json_object"}
+            model.structured_outputs = False
 
     def _search_user_memories_agentic(self, user_id: str, query: str, limit: Optional[int] = None) -> List[UserMemory]:
-        """Search through user memories using semantic search."""
+        """Search through user memories using agentic search."""
         if not self.memories:
             return []
 
