@@ -125,8 +125,7 @@ class AwsBedrock(Model):
             "stopSequences": self.stop_sequences,
         }
 
-        request_kwargs = {k: v for k, v in request_kwargs.items() if v is not None}
-        return request_kwargs
+        return {k: v for k, v in request_kwargs.items() if v is not None}
 
     def _format_messages(self, messages: List[Message]) -> Tuple[List[Dict[str, Any]], Optional[List[Dict[str, Any]]]]:
         formatted_messages: List[Dict[str, Any]] = []
@@ -135,8 +134,7 @@ class AwsBedrock(Model):
             if message.role == "system":
                 system_message = [{"text": message.content}]
             else:
-                formatted_message: Dict[str, Any] = {"role": message.role}
-                formatted_message["content"] = []
+                formatted_message: Dict[str, Any] = {"role": message.role, "content": []}
                 # Handle tool results
                 if isinstance(message.content, list):
                     formatted_message["content"].extend(message.content)
@@ -158,55 +156,56 @@ class AwsBedrock(Model):
 
                 if message.images:
                     for image in message.images:
-                        # Only supported via bytes for now
-                        if image.content and image.format:
-                            if image.format not in ["png", "jpeg", "webp", "gif"]:
-                                raise ValueError(f"Unsupported image format: {image.format}")
-
-                            formatted_message["content"].append(
-                                {
-                                    "image": {
-                                        "format": image.format,
-                                        "source": {
-                                            "bytes": image.content,
-                                        },
-                                    }
-                                }
-                            )
-                        else:
+                        if not image.content or not image.format:
                             raise ValueError("Image content and format are required.")
 
+                        if image.format not in ["png", "jpeg", "webp", "gif"]:
+                            raise ValueError(f"Unsupported image format: {image.format}")
+
+                        formatted_message["content"].append(
+                            {
+                                "image": {
+                                    "format": image.format,
+                                    "source": {
+                                        "bytes": image.content,
+                                    },
+                                }
+                            }
+                        )
                 if message.audio:
                     log_warning("Audio input is currently unsupported.")
 
                 if message.videos:
                     for video in message.videos:
-                        if video.content and video.format:
-                            if video.format not in [
-                                "mp4",
-                                "mov",
-                                "mkv",
-                                "webm",
-                                "flv",
-                                "mpeg",
-                                "mpg",
-                                "wmv",
-                                "three_gp",
-                            ]:
-                                raise ValueError(f"Unsupported video format: {video.format}")
-
-                            formatted_message["content"].append(
-                                {
-                                    "video": {
-                                        "format": video.format,
-                                        "source": {
-                                            "bytes": video.content,
-                                        },
-                                    }
-                                }
-                            )
-                        else:
+                        if not video.content or not video.format:
                             raise ValueError("Video content and format are required.")
+
+                        if video.format not in [
+                            "mp4",
+                            "mov",
+                            "mkv",
+                            "webm",
+                            "flv",
+                            "mpeg",
+                            "mpg",
+                            "wmv",
+                            "three_gp",
+                        ]:
+                            raise ValueError(f"Unsupported video format: {video.format}")
+
+                        formatted_message["content"].append(
+                            {
+                                "video": {
+                                    "format": video.format,
+                                    "source": {
+                                        "bytes": video.content,
+                                    },
+                                }
+                            }
+                        )
+                if message.files is not None and len(message.files) > 0:
+                    log_warning("File input is currently unsupported.")
+
                 formatted_messages.append(formatted_message)
         # TODO: Add caching: https://docs.aws.amazon.com/bedrock/latest/userguide/conversation-inference-call.html
         return formatted_messages, system_message
@@ -223,7 +222,6 @@ class AwsBedrock(Model):
         """
         try:
             formatted_messages, system_message = self._format_messages(messages)
-            inference_config = self._get_inference_config()
 
             tool_config = None
             if self._functions is not None:
@@ -232,10 +230,13 @@ class AwsBedrock(Model):
             body = {
                 "system": system_message,
                 "toolConfig": tool_config,
-                "inferenceConfig": inference_config,
+                "inferenceConfig": self._get_inference_config(),
             }
             body = {k: v for k, v in body.items() if v is not None}
 
+            if self.request_params:
+                body.update(**self.request_params)
+
             return self.get_client().converse(modelId=self.id, messages=formatted_messages, **body)
         except ClientError as e:
             log_error(f"Unexpected error calling Bedrock API: {str(e)}")
@@ -256,7 +257,6 @@ class AwsBedrock(Model):
         """
         try:
             formatted_messages, system_message = self._format_messages(messages)
-            inference_config = self._get_inference_config()
 
             tool_config = None
             if self._functions is not None:
@@ -265,10 +265,13 @@ class AwsBedrock(Model):
             body = {
                 "system": system_message,
                 "toolConfig": tool_config,
-                "inferenceConfig": inference_config,
+                "inferenceConfig": self._get_inference_config(),
             }
             body = {k: v for k, v in body.items() if v is not None}
 
+            if self.request_params:
+                body.update(**self.request_params)
+
             return self.get_client().converse_stream(modelId=self.id, messages=formatted_messages, **body)["stream"]
         except ClientError as e:
             log_error(f"Unexpected error calling Bedrock API: {str(e)}")
@@ -289,7 +292,7 @@ class AwsBedrock(Model):
             function_call_results (List[Message]): The results of the function calls.
             tool_ids (List[str]): The tool ids.
         """
-        if len(function_call_results) > 0:
+        if function_call_results:
             tool_result_content: List = []
 
             for _fc_message_index, _fc_message in enumerate(function_call_results):
