@@ -7,3 +7,9 @@ except ImportError:
     class AzureOpenAI:  # type: ignore
         def __init__(self, *args, **kwargs):
             raise ImportError("`openai` not installed. Please install it via `pip install openai`")
+
+
+__all__ = [
+    "AzureAIFoundry",
+    "AzureOpenAI",
+]
