@@ -61,6 +61,9 @@ def _format_message(message: Message) -> Dict[str, Any]:
     if message.audio is not None and len(message.audio) > 0:
         log_warning("Audio input is currently unsupported.")
 
+    if message.files is not None and len(message.files) > 0:
+        log_warning("File input is currently unsupported.")
+
     if message.videos is not None and len(message.videos) > 0:
         log_warning("Video input is currently unsupported.")
 
