@@ -1,6 +1,5 @@
 import json
 import time
-import traceback
 from dataclasses import dataclass
 from os import getenv
 from pathlib import Path
@@ -53,7 +52,7 @@ class Gemini(Model):
     Based on https://googleapis.github.io/python-genai/
     """
 
-    id: str = "gemini-2.0-flash-exp"
+    id: str = "gemini-2.0-flash-001"
     name: str = "Gemini"
     provider: str = "Google"
 
@@ -79,6 +78,7 @@ class Gemini(Model):
     seed: Optional[int] = None
     response_modalities: Optional[list[str]] = None  # "Text" and/or "Image"
     speech_config: Optional[dict[str, Any]] = None
+    cached_content: Optional[Any] = None
     request_params: Optional[Dict[str, Any]] = None
 
     # Client parameters
@@ -168,6 +168,7 @@ class Gemini(Model):
                 "seed": self.seed,
                 "response_modalities": self.response_modalities,
                 "speech_config": self.speech_config,
+                "cached_content": self.cached_content,
             }
         )
 
@@ -213,6 +214,7 @@ class Gemini(Model):
         # Filter out None values
         if self.request_params:
             request_params.update(self.request_params)
+
         return request_params
 
     def invoke(self, messages: List[Message]):
@@ -326,6 +328,7 @@ class Gemini(Model):
             messages (List[Message]): The list of messages to convert.
         """
         formatted_messages: List = []
+        file_content: Union[GeminiFile, Part] = None
         system_message = None
         for message in messages:
             role = message.role
@@ -393,7 +396,6 @@ class Gemini(Model):
                                 if video_file is not None:
                                     message_parts.insert(0, video_file)  # type: ignore
                     except Exception as e:
-                        traceback.print_exc()
                         log_warning(f"Failed to load video from {message.videos}: {e}")
                         continue
 
@@ -421,11 +423,15 @@ class Gemini(Model):
                 if message.files is not None:
                     for file in message.files:
                         file_content = self._format_file_for_message(file)
-                        if file_content:
-                            message_parts.append(file_content)
+                        if isinstance(file_content, Part):
+                            formatted_messages.append(file_content)
 
             final_message = Content(role=role, parts=message_parts)
             formatted_messages.append(final_message)
+
+            if isinstance(file_content, GeminiFile):
+                formatted_messages.insert(0, file_content)
+
         return formatted_messages, system_message
 
     def _format_audio_for_message(self, audio: Audio) -> Optional[Union[Part, GeminiFile]]:
@@ -485,7 +491,7 @@ class Gemini(Model):
             log_warning(f"Unknown audio type: {type(audio.content)}")
             return None
 
-    def _format_video_for_message(self, video: Video) -> Optional[GeminiFile]:
+    def _format_video_for_message(self, video: Video) -> Optional[Part]:
         # Case 1: Video is a bytes object
         if video.content and isinstance(video.content, bytes):
             return Part.from_bytes(
@@ -532,6 +538,12 @@ class Gemini(Model):
             return Part.from_uri(
                 file_uri=video_file.uri, mime_type=f"video/{video.format}" if video.format else "video/mp4"
             )
+        # Case 3: Video is a URL
+        elif video.url is not None:
+            return Part.from_uri(
+                file_uri=video.url,
+                mime_type=f"video/{video.format}" if video.format else "video/webm",
+            )
         else:
             log_warning(f"Unknown video type: {type(video.content)}")
             return None
@@ -565,19 +577,35 @@ class Gemini(Model):
                             mime_type=mimetypes.guess_type(file_path)[0], data=file_path.read_bytes()
                         )
                 else:
-                    file_upload = self.get_client().files.upload(
-                        file=file_path,
-                    )
-                    # Check whether the file is ready to be used.
-                    while file_upload.state.name == "PROCESSING":
-                        time.sleep(2)
-                        file_upload = self.get_client().files.get(name=file_upload.name)
-                    if file_upload.state.name == "FAILED":
-                        raise ValueError(file_upload.state.name)
-                    return Part.from_uri(file_uri=file_upload.uri, mime_type=file_upload.mime_type)
+                    clean_file_name = f"files/{file_path.stem.lower().replace('_', '')}"
+                    remote_file = None
+                    try:
+                        remote_file = self.get_client().files.get(name=clean_file_name)
+                    except Exception as e:
+                        log_warning(f"Error getting file {clean_file_name}: {e}")
+
+                    if remote_file is not None:
+                        return Part.from_uri(file_uri=remote_file.uri, mime_type=remote_file.mime_type)
+                    else:
+                        file_upload = self.get_client().files.upload(
+                            file=file_path,
+                            config=dict(
+                                name=clean_file_name,
+                            ),
+                        )
+                        # Check whether the file is ready to be used.
+                        while file_upload.state.name == "PROCESSING":
+                            time.sleep(2)
+                            file_upload = self.get_client().files.get(name=file_upload.name)
+                        if file_upload.state.name == "FAILED":
+                            raise ValueError(file_upload.state.name)
+                        return Part.from_uri(file_uri=file_upload.uri, mime_type=file_upload.mime_type)
             else:
                 log_error(f"File {file_path} does not exist.")
 
+        # Case 4: File is a Gemini File object
+        elif isinstance(file.external, GeminiFile):
+            return file.external
         return None
 
     def format_function_call_results(
@@ -636,7 +664,9 @@ class Gemini(Model):
 
                     # Extract function call if present
                     if hasattr(part, "function_call") and part.function_call is not None:
+                        call_id = part.function_call.id if part.function_call.id else str(uuid4())
                         tool_call = {
+                            "id": call_id,
                             "type": "function",
                             "function": {
                                 "name": part.function_call.name,
@@ -673,6 +703,7 @@ class Gemini(Model):
                 "input_tokens": usage.prompt_token_count or 0,
                 "output_tokens": usage.candidates_token_count or 0,
                 "total_tokens": usage.total_token_count or 0,
+                "cached_tokens": usage.cached_content_token_count or 0,
             }
 
         return model_response
@@ -699,7 +730,9 @@ class Gemini(Model):
 
                 # Extract function call if present
                 if hasattr(part, "function_call") and part.function_call is not None:
+                    call_id = part.function_call.id if part.function_call.id else str(uuid4())
                     tool_call = {
+                        "id": call_id,
                         "type": "function",
                         "function": {
                             "name": part.function_call.name,
@@ -736,6 +769,49 @@ class Gemini(Model):
                 "input_tokens": usage.prompt_token_count or 0,
                 "output_tokens": usage.candidates_token_count or 0,
                 "total_tokens": usage.total_token_count or 0,
+                "cached_tokens": usage.cached_content_token_count or 0,
             }
 
         return model_response
+
+    def __deepcopy__(self, memo):
+        """
+        Creates a deep copy of the Gemini model instance but sets the client to None.
+
+        This is useful when we need to copy the model configuration without duplicating
+        the client connection.
+
+        This overrides the base class implementation.
+        """
+        from copy import copy, deepcopy
+
+        # Create a new instance without calling __init__
+        cls = self.__class__
+        new_instance = cls.__new__(cls)
+
+        # Update memo with the new instance to avoid circular references
+        memo[id(self)] = new_instance
+
+        # Deep copy all attributes except client and unpickleable attributes
+        for key, value in self.__dict__.items():
+            # Skip client and other unpickleable attributes
+            if key in {"client", "response_format", "_tools", "_functions", "_function_call_stack"}:
+                continue
+
+            # Try deep copy first, fall back to shallow copy, then direct assignment
+            try:
+                setattr(new_instance, key, deepcopy(value, memo))
+            except Exception:
+                try:
+                    setattr(new_instance, key, copy(value))
+                except Exception:
+                    setattr(new_instance, key, value)
+
+        # Explicitly set client to None
+        setattr(new_instance, "client", None)
+
+        # Clear the new model to remove any references to the old model
+        if hasattr(new_instance, "clear"):
+            new_instance.clear()
+
+        return new_instance
