@@ -173,6 +173,16 @@ class Ollama(Model):
                         message_images.append(image.content)
                 if message_images:
                     _message["images"] = message_images
+
+            if message.audio is not None and len(message.audio) > 0:
+                log_warning("Audio input is currently unsupported.")
+
+            if message.files is not None and len(message.files) > 0:
+                log_warning("File input is currently unsupported.")
+
+            if message.videos is not None and len(message.videos) > 0:
+                log_warning("Video input is currently unsupported.")
+
         return _message
 
     def _prepare_request_kwargs_for_invoke(self) -> Dict[str, Any]:
