@@ -325,7 +325,7 @@ def get_sync_playground_router(
                     else:
                         raise HTTPException(status_code=400, detail="Unsupported file type")
 
-        if stream:
+        if stream and agent.is_streamable:
             return StreamingResponse(
                 chat_response_streamer(
                     agent,
@@ -692,7 +692,7 @@ def get_sync_playground_router(
                 else:
                     raise HTTPException(status_code=400, detail="Unsupported file type")
 
-        if stream:
+        if stream and team.is_streamable:
             return StreamingResponse(
                 team_chat_response_streamer(
                     team,
