@@ -1 +1,5 @@
 from agno.vectordb.chroma.chromadb import ChromaDb
+
+__all__ = [
+    "ChromaDb",
+]
