@@ -1 +1,5 @@
 from agno.vectordb.milvus.milvus import Milvus
+
+__all__ = [
+    "Milvus",
+]
