@@ -1 +1,5 @@
 from agno.vectordb.mongodb.mongodb import MongoDb
+
+__all__ = [
+    "MongoDb",
+]
